require 'spec_helper'

describe Kurki do

  before :each do

    stub_request(:get, /.*courses.*/).
        with(:headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => IO.read("spec/fixtures/courses.json"), :headers => {})

    stub_request(:get, /.*courses\/581259.2012.K.K.1/).
        with(:headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => IO.read("spec/fixtures/course.json"), :headers => {})

    stub_request(:get, /.*courses\/581259.2012.K.K.1\/students.*/).
        with(:headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => IO.read("spec/fixtures/students.json"), :headers => {})

    stub_request(:post, /.*581259.2012.K.K.1\/students.*/).
        with(:body => "{\"students\":[{\"id\":\"012345678\"},{\"id\":\"01293492\"}]}",
             :headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Length'=>'51', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => "{\"success\":[{\"id\":\"012345678\"},{\"id\":\"01293492\"}],\"error\":[]}", :headers => {})

    stub_request(:post, /.*581259.2012.K.K.1\/students.*/).
        with(:body => "{\"students\":[{\"id\":\"012345678\",\"arvosana\":\"4\"}]}",
             :headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Length'=>'48', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => "{\"success\":[{\"id\":\"012345678\",\"arvosana\":\"4\"}],\"error\":[]}", :headers => {})

    stub_request(:post, /.*581259.2012.K.K.1\/students.*/).
        with(:body => "{\"students\":[{\"id\":\"123123123\",\"arvosana\":\"3\"},{\"id\":\"014475359\",\"arvosana\":\"5\"},{\"id\":\"014291915\",\"arvosana\":\"1\"},{\"id\":\"123456789\",\"arvosana\":\"0\"}]}",
             :headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Length'=>'150', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => "{\"success\":[{\"id\":\"123123123\",\"arvosana\":\"3\"},
                          {\"id\":\"014475359\",\"arvosana\":\"5\"},
                          {\"id\":\"014291915\",\"arvosana\":\"1\"},
                          {\"id\":\"123456789\",\"arvosana\":\"0\"}],\"error\":[]}", :headers => {})

    stub_request(:post, /.*581259.2012.K.K.1\/students.*/).
        with(:body => "{\"students\":[{\"id\":\"01293492\",\"exams\":{\"2\":23,\"1\":11}},{\"id\":\"012312333\",\"exams\":{\"1\":22,\"2\":40}}]}",
             :headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Length'=>'99', 'Content-Type'=>'application/json'}).
        to_return(:status => 200, :body => "{\"success\":[
                                              {\"id\":\"01293492\",\"exams\":{\"2\":23, \"1\":11}},
                                              {\"id\":\"012312333\",\"exams\":{\"1\":22, \"2\":40}}
                                            ],
                                            \"error\":[]}", :headers => {})

     stub_request(:post, /.*581259.2012.K.K.1\/students.*/).
          with(:body => "{\"students\":[{\"id\":\"014475359\",\"exercises\":{\"2\":23,\"1\":11}},{\"id\":\"012312333\",\"exercises\":{\"1\":22,\"2\":40}}]}",
               :headers => {'Accept'=>'application/json', 'Accept-Encoding'=>'gzip, deflate', 'Content-Length'=>'108', 'Content-Type'=>'application/json', 'Host'=>'test'}).
          to_return(:status => 200, :body => "{\"success\":[
          {\"id\":\"014475359\", \"exercises\":{\"2\":23, \"1\":11}}],
           \"error\":[{\"id\":\"012312333\", \"exercises\":{\"1\":22, \"2\":40}}]}", :headers => {})


  end

  describe "GET" do

    it "setting an url without a backslash adds it" do
      ENV["KURKI_URL"] = "test"
      expect(Kurki.url).to eq("test/")
    end

    it "default url works" do
      expect(Kurki.url).not_to be_nil
    end

    it 'can GET the curses index' do
      courses = Kurki.get_courses
      expect(courses.first["kurssikoodi"]).to eq("581259")
      expect(courses.last["kurssikoodi"]).to eq("58131")
    end

    it 'can GET the a single course by id' do
      course = Kurki.get_course("581259.2012.K.K.1")
      expect(course["kurssikoodi"]).to eq("581259")
    end

    it 'can get GET students of a course' do
      students = Kurki.get_students("581259.2012.K.K.1")
      expect(students.first["arvosana"]).to eq("3")
    end

  end

  describe "POST" do

    it "can parse student array correctly" do
      hash = Kurki.parse_students(["01293492","18239123","812383172"])
      expect(hash).to have_key("students")
      expect(hash["students"].size).to eq(3)
      expect(hash["students"].first).to eq({id: "01293492"})
    end

    it "can parse grades array correctly" do
      hash = Kurki.parse_grades({"01293492" => "3","18239123" => "4","812383172" => "1"})
      expect(hash).to have_key("students")
      expect(hash["students"].size).to eq 3
      expect(hash["students"].is_a? Array)
      expect(hash["students"].first).to eq({id:"01293492", arvosana: "3"})
    end

    it "can POST students to a course" do
      students = ["012345678", "01293492"]
      ans = Kurki.set_students("581259.2012.K.K.1", students)
      expect(ans).to eq(
                         JSON.parse("{\"success\":[{\"id\":\"012345678\"},{\"id\":\"01293492\"}],\"error\":[]}"))
    end

    it "can POST grades from an array" do
      grades = {"012345678" => "4"}
      ans = Kurki.set_grades("581259.2012.K.K.1", grades)
      expect(ans).to eq(
                         JSON.parse("{\"success\":[{\"id\":\"012345678\",\"arvosana\":\"4\"}],\"error\":[]}"))
    end

    it "can POST grades from a CSV file" do
      ans = Kurki.set_grades_from_csv("581259.2012.K.K.1", "spec/fixtures/grades.csv")
      expect(ans).to eq(
                          JSON.parse("{\"success\":[{\"id\":\"123123123\",\"arvosana\":\"3\"},
                          {\"id\":\"014475359\",\"arvosana\":\"5\"},
                          {\"id\":\"014291915\",\"arvosana\":\"1\"},
                          {\"id\":\"123456789\",\"arvosana\":\"0\"}],\"error\":[]}"
                         )
                     )
    end

    it "can parse exam points correctly" do
      hash = Kurki.parse_exam_points({"01293492" => {"2" =>23, "1"=>11}, "012312333" => {"1"=>22, "2"=>40} })
      expect(hash).to have_key("students")
      expect(hash["students"].size).to eq 2
      expect(hash["students"].is_a? Array)
      expect(hash["students"].first).to have_key(:id)
      expect(hash["students"].first[:id]).to eq("01293492")
      expect(hash["students"][1][:exams]).to eq({"1"=>22, "2"=>40})
    end

    it "can POST exam points from a hash" do
      exam_points = {"01293492" => {"2" =>23, "1"=>11}, "012312333" => {"1"=>22, "2"=>40} }
      ans = Kurki.set_exam_points("581259.2012.K.K.1", exam_points)
      expect(ans). to eq(
                          JSON.parse("{\"success\":[
                            {\"id\":\"01293492\",\"exams\":{\"2\":23, \"1\":11}},
                            {\"id\":\"012312333\",\"exams\":{\"1\":22, \"2\":40}}
                           ],
                           \"error\":[]}" )
                      )
    end

    it "can POST exercise points from a hash" do
      exercise_points = {"014475359" => {"2" =>23, "1"=>11}, "012312333" => {"1"=>22, "2"=>40} }
      ans = Kurki.set_exercise_points("581259.2012.K.K.1", exercise_points)
      resp = "{\"success\":[
      {\"id\":\"014475359\", \"exercises\":{\"2\":23, \"1\":11}}],
       \"error\":[{\"id\":\"012312333\", \"exercises\":{\"1\":22, \"2\":40}}]}"
      expect(ans). to eq(JSON.parse(resp))
    end

  end
end
